(function ($) {
    Drupal.behaviors.bynderMedia = {
        attach: function () {
            'use strict';

            var spinner;

            var showAlert = function (message, type) {
                var alert = $('#edit-bynder-search .alert');
                alert.find('span.text').text(message);
                alert.removeClass('alert-warning').removeClass('alert-success').removeClass('alert-error');
                alert.addClass('alert-' + type);
                alert.fadeIn();
                setTimeout(function () {
                    alert.fadeOut(1000);
                }, 2500);
            };

            var showSpinner = function (element) {
                spinner = new Spinner({
                    lines: 13,
                    length: 20,
                    width: 10,
                    radius: 30,
                    corners: 1,
                    rotate: 0,
                    direction: 1,
                    color: '#000',
                    speed: 1,
                    trail: 60,
                    shadow: true,
                    hwaccel: false,
                    className: 'spinner',
                    zIndex: 2e9,
                    top: '50%',
                    left: '50%'
                }).spin(element);
            };

            // Public variables/methods
            var isValid = function (type, val) {
                switch (type) {
                    case "uuid":
                        return /^[0-9a-fA-F]{8}-?[0-9a-fA-F]{4}-?[0-9a-fA-F]{4}-?[0-9a-fA-F]{4}-?[0-9a-fA-F]{12}$/.test(val);
                    case "idHash":
                        return /[a-f0-9]{16}/.test(val);
                }
                return true;
            };

            $(document).ready(function () {
                var $grid = $('.grid');
                $grid.prepend('<div class="grid-sizer"></div><div class="gutter-sizer"></div>').once();
                // layout Masonry after each image loads
                $grid.imagesLoaded().progress( function() {
                    $grid.masonry({
                        columnWidth: '.grid-sizer',
                        gutter: '.gutter-sizer',
                        itemSelector: '.grid-item'
                    });
                });

                $('.grid-item').once('bynder-bind-click-event').click(function () {
                    var $input = $(this).find('.item-selector');
                    $input.attr('checked', !$input.attr('checked'));
                    if ($input.attr('checked')) {
                        $(this).addClass('checked');
                    }
                    else {
                        $(this).removeClass('checked');
                    }
                });

                $('#edit-bynder-search .normal_facet_list > .facet_title').click(function () {
                    var $filters = $(this).siblings();
                    $filters.find('.expand i.fa-angle-up')
                        .addClass('fa-angle-down')
                        .removeClass('fa-angle-up');
                    $filters.find('.item-list:visible').slideUp(300);

                    $(this).find('.expand i').toggleClass('fa-angle-down').toggleClass('fa-angle-up');
                    $(this).find(' > .item-list').slideToggle(300);
                });


                $('#edit-bynder-search .selected_facet_list > .facet_title').click(function (e) {
                    var link = $(e.currentTarget);
                    var filters_input = $('#edit-bynder-search input[name="filters"]');

                    var filter_key = link.data('filter-key');
                    var current_filters = JSON.parse(filters_input.val() || '{}');
                    current_filters.filters = current_filters.filters || [];

                    var exists = ($.grep(current_filters.filters, function (e, i) {
                        return e.key == filter_key;
                    }).length);
                    if (exists) {
                        //Remove current filter from list
                        current_filters.filters = $.grep(current_filters.filters, function (e, i) {
                            return e.key != filter_key;
                        });
                        link.removeClass('active');
                    }
                    filters_input.val(JSON.stringify(current_filters));
                    link.parent().fadeOut(300, function () {
                        $(this).remove();
                    });

                    var input = $('<input>', {
                        type: 'hidden',
                        name: 'source_filter',
                        value: $(this).attr('data-filter-key')
                    });
                    $('#bynder-add').append($(input));
                    if ($(this).attr('data-filter-key') == 'bynder_remove_all_facets_filter_key') {
                        $('#edit-bynder-search input[name="filters"]').val('{"filters":[]}');
                    }

                    $('#bynder-add').submit();
                });

                $('#edit-bynder-search .filter-url').click(function (e) {
                    e.preventDefault();

                    //go back to the first page
                    var old_form_action = $('#bynder-add').attr('action');
                    var new_form_action = old_form_action.replace(/(page=)[^\&]+/, '$1' + '0');
                    $('#bynder-add').attr('action', new_form_action);

                    var link = $(e.currentTarget);
                    var filter_key = link.data('filter-key');
                    var filter_value = link.data('filter-value');

                    var filters_input = $('#edit-bynder-search input[name="filters"]');

                    var current_filters = JSON.parse(filters_input.val() || '{}');
                    current_filters.filters = current_filters.filters || [];

                    var exists = ($.grep(current_filters.filters, function (e, i) {
                        return e.key == filter_key;
                    }).length);
                    if (exists) {
                        //Remove current filter from list
                        current_filters.filters = $.grep(current_filters.filters, function (e, i) {
                            return e.key != filter_key;
                        });
                        link.removeClass('active');
                    }

                    // Add current filter to list
                    current_filters.filters.push({
                        key: filter_key,
                        value: filter_value,
                    });
                    link.addClass('active');

                    filters_input.val(JSON.stringify(current_filters));
                    $('#bynder-add').submit();
                });

                $('#bynder-add').submit(function (e) {
                    $('#bynder-add').prepend('<div class="overlay-throbber"><div class="throbber-spinner"></div></div></div>');
                    var elem = $('.throbber-spinner');
                    $(elem).addClass('spinning');
                    showSpinner($(elem)[0]);
                });
            });
        }
    }
})(jQuery);

